%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   simar_annualshocks.m
%   SIMULATE STATES AND FIT OUT/INC REGRESSIONS
%   ANNUAL SHOCKS TO PRODUCTIVITY
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear
clc

tic
disp('simar_annualshocks.m')
c=(fix(clock)); hrs=num2str(c(4)); min=num2str(c(5)); sec=num2str(c(6));
disp([date ', ' hrs ':' min ':' sec])
clear c hrs min sec

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   PARAMETERS
itermax=200 ;       % number of simulations
states=50 ;         % number of states
tmax=100 ;          % number of periods in each simulation
global s delta alpha autocoef sigmaA L
s=0.2  ;           % saving rate
delta=0.07 ;        % depreciation rate
alpha=1/3 ;         % capital's share
autocoef=.99 ;      % autocorrelation coef for productivity shocks
sigmaA=0.02 ;       % stdev of productivity shocks
L=ones(states,1) ;  % labor force
global A Kworld     % will be used in simulations below
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   ITERATIONS OF SIMULATIONS

options = optimset('Display','off') ;
%options = optimset('LargeScale','off','TolFun', 0.00001, 'TolX', 0.0001,...
%    'MaxIter',100000,'MaxFunEvals',1000000) ;

level1=zeros(itermax,2) ;   % ln level on ln lagged output
level2=zeros(itermax,3);    % ln level on ln lagged output and ln lagged ownership
change1=zeros(itermax,2) ;  % change on lagged growth
change2=zeros(itermax,3) ;  % change on lagged growth and lagged ratio
level1sdv=zeros(itermax,2) ;
level2sdv=zeros(itermax,3) ;
change1sdv=zeros(itermax,2) ;
change2sdv=zeros(itermax,3) ;


for iter=1:1:itermax
    disp(iter)
        
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %   SIMULATION
    
    out=zeros(states,tmax) ;        % state output (GDP)
    inc=zeros(states,tmax) ;        % state income (GNI)
    shown=zeros(states,tmax+1) ;    % ownership shares
    r=zeros(1,tmax) ;               % interest rates
    %   initial values for simulation
    shown(:,1)=(1/states)*ones(states,1) ; % symetric ownership shares
    A0=1 ;              % steady state productivity levels
    A=A0*ones(states,1);% initial level of state productivities
    Kworld=states ;     % initial world capital stock
    
    for t=1:1:tmax
        r(:,t)=fsolve(@simar_equilibrium,0.1,options) ; % solve for int. rate
        K=L.*(alpha*A/r(:,t)).^(1/(1-alpha)) ;  % back out state capital stocks
        out(:,t)=A.*K.^alpha.*L.^(1-alpha) ;    % state outputs (GSP)
        wagebill=(1-alpha)*out(:,t) ;           % state wage bill
        inc(:,t)=wagebill+shown(:,t).*r(:,t).*Kworld ;  % state incomes
        own=s.*inc(:,t)+shown(:,t).*(1-delta).*Kworld ; % stock of owned capital in world mutual fund
        % values for next iteration
        Kworld=sum(own) ;
        A=A0.^(1-autocoef).*exp(sigmaA*randn(states,1)).*A.^(autocoef) ;
        shown(:,t+1)=own./sum(own) ;
    end
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %   REGRESSIONS
    
    %   define variables for regressions
    outinc=out./inc ;
    const=ones(states,1) ;
    meanratio=mean(outinc(:,tmax-20:tmax),2) ;
    meanlaggedoutput=mean(out(:,tmax-23:tmax-21),2) ;
    meanlaggedgrowth=log(out(:,tmax-10))-log(out(:,tmax-20)) ;
    meanchangeratio=mean(outinc(:,tmax-9:tmax),2)-mean(outinc(:,tmax-19:tmax-10),2) ;
    meanlaggedratio=mean(outinc(:,tmax-19:tmax-10),2) ;
    meanlaggedownership=mean(shown(:,tmax-61:tmax-51),2) ; 
    
    %   level 1
    X=[const log(meanlaggedoutput)] ;
    level1(iter,:)=(inv(X'*X)*X'*log(meanratio))' ;
    res=log(meanratio) - X*level1(iter,:)';
    sighat=res'*res/(length(meanratio)-length(level1)) ;
    glsvar = sighat*inv(X'*X) ;
    level1sdv(iter,:) = sqrt(diag(glsvar))' ; % standard deviations
    
    %   level 2
    X=[const log(meanlaggedoutput) log(meanlaggedownership)] ;
    level2(iter,:)=(inv(X'*X)*X'*log(meanratio))' ;
    res=log(meanratio) - X*level2(iter,:)';
    sighat=res'*res/(length(meanratio)-length(level2)) ;
    glsvar = sighat*inv(X'*X) ;
    level2sdv(iter,:) = sqrt(diag(glsvar))' ; % standard deviations
    
    %   change 1
    X=[const meanlaggedgrowth] ;
    change1(iter,:)=(inv(X'*X)*X'*meanchangeratio)' ; % change on lagged growth
    res=meanchangeratio-X*change1(iter,:)';
    sighat=res'*res/(length(meanchangeratio)-length(change1)) ;
    glsvar = sighat*inv(X'*X) ;
    change1sdv(iter,:) = sqrt(diag(glsvar))' ;
    
    %   change 2
    X=[const meanlaggedgrowth meanlaggedratio] ;
    change2(iter,:)=(inv(X'*X)*X'*meanchangeratio)' ; % change on lagged growth
    res=meanchangeratio-X*change2(iter,:)';
    sighat=res'*res/(length(meanchangeratio)-length(change2)) ;
    glsvar = sighat*inv(X'*X) ;
    change2sdv(iter,:) = sqrt(diag(glsvar))' ;
    
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   RESULTS

disp(' ')
disp('PARAMETERS')
disp(' ')

disp('saving rate:')
disp(s)
disp('depreciation rate:')
disp(delta)
disp('capital share:')
disp(alpha)
disp('autocorrelation coef. of state TFP:')
disp(autocoef)
disp('stdev of AR(1) innovation of state TFP:')
disp(sigmaA)

%disp(' ')
%disp('INITIAL VALUES')
%disp(' ')
%disp('World capital:')
%disp(Kworld)
%disp('Symmetric labor:')
%disp(L(1))
%disp('Initial, steady state productivity:')
%disp(A0)

disp(' ')
disp('REGRESSION RESULTS')
disp(' ')
disp('First row is average of regression results')
disp('Second row is stdev of regression results')
disp(' ')

disp('level 1: reg log(meanratio) const log(meanlaggedoutput)')
disp([mean(level1) ; std(level1)])

disp('level 2: reg log(meanratio) const log(meanlaggedoutput) log(meanlaggedownership)')
disp([mean(level2) ; std(level2)])

disp('change 1: reg meanchangeratio const meanlaggedgrowth')
disp([mean(change1) ; std(change1)])

disp('change 2: reg meanchangeratio const meanlaggedgrowth meanlaggedratio')
disp([mean(change2) ; std(change2)])

disp(' ')
disp('DATA MOMENTS FOR LAST 20 YEAR')
disp(' ')

disp('average and stdev interest rate:')
disp([mean(r(end-19:end)) std(r(end-19:end))])

GDP = sum(out(:,end-19:end)) ;
disp('stdev and autocorrelation of ln(GDP):')
c=corr([log(GDP(end-18:end))' log(GDP(end-19:end-1))']);
disp([std(log(GDP)) c(2,1)])

disp('average and median stdev of OUTINC:')
disp([mean(std(outinc(:,end-19:end))) median(std(outinc(:,end-19:end)))])

correl=zeros(states,1);
for i=1:1:states
    c=corr([log(outinc(i,end-18:end))' log(outinc(i,end-19:end-1))']) ;
    correl(i,1)=c(2,1) ;
end

disp('average and median autocorrelation of OUTINC:')
disp([mean(correl) median(correl)])

disp('average and median stdev of log OUT:')
disp([mean(std(log(out(:,end-19:end)))) median(std(log(out(:,end-19:end))))])

correl=zeros(states,1);
for i=1:1:states
    c=corr([log(out(i,end-18:end))' log(out(i,end-19:end-1))']) ;
    correl(i,1)=c(2,1) ;
end

disp('average and median autocorrelation of log OUT:')
disp([mean(correl) median(correl)])

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


disp('done!')
toc